int release(uint8_t R, struct in6_addr *server_addr, struct in6_addr *x_addr,
	    IP_ADDR_EXT *ip_ext_head) {
  /*
   * This function is a part of library which applications 
   * will use.
   * Sending initialization message to client daemon on
   * behalf of the application. This would send
   *
   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   * |   msg_type    | error(8 bits) |        pid (16 bits)          |
   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   * |                   request_no (32 bits)                        |
   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   * |             Reserved (31 bits)                              |R|
   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   * |                                                               |
   * |                server-address (16 octets)                     |
   * |                                                               |
   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   * |                                                               |
   * |                   x-address (16 octets)                       |
   * |                                                               |
   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   * |              extensions (variable number and length)          |
   * +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   */

  
  typedef struct type_len_settings {
    uint16_t type;
    uint16_t length;
    uint32_t settings;
    struct type_len_settings *next;
  } TYPE_LEN_SETTINGS;

  HEADER_CLIENTD_PACKET  header;
  uint32_t               reserved_r;
  struct iovec           iov[128];
  TYPE_LEN_SETTINGS     *tls_head = NULL, **temp_tls = &tls_head, *t_tls;
  int i = 0, len = 0;

  /* Check validity of parameters passed to this function */
  if(R > 1){
#ifdef DEBUG
    printf("Invalid parameters to release by the appl\n");
#endif
    return 0;
  }

  header.error      = SUCCESS;
  header.msg_type   = DHCP_RELEASE;
  header.pid        = htons(getpid());
  header.request_no = htonl(request_no++);

  iov[i].iov_base   = (char *) &header;
  iov[i].iov_len    = sizeof(header);
  len              += iov[i++].iov_len;
  
  reserved_r        = htonl(R);

  iov[i].iov_base   = (char *) &reserved_r;
  iov[i].iov_len    = sizeof(reserved_r);
  len              += iov[i++].iov_len;

  if(!server_addr || IN6_IS_ADDR_UNSPECIFIED(server_addr)) {
    printf("Application sent NULL or unspecified server address to "
	   "library function release. Returning...\n");
    return 0;
  }
  
  iov[i].iov_base   = (char *) server_addr;
  iov[i].iov_len    = sizeof(*server_addr);
  len              += iov[i++].iov_len;

  if(!x_addr || IN6_IS_ADDR_UNSPECIFIED(x_addr)) {
    printf("Application sent NULL or unspecified x address to "
	   "library function release. Returning...\n");
    return 0;
  }

  iov[i].iov_base   = (char *) x_addr;
  iov[i].iov_len    = sizeof(*x_addr);
  len              += iov[i++].iov_len;

  
  /* Attach IP address extensions */
  for(; ip_ext_head; ip_ext_head = ip_ext_head->next) {
    t_tls = (TYPE_LEN_SETTINGS *) malloc(sizeof(TYPE_LEN_SETTINGS));
    if(!t_tls)
      errx(1, "Insufficient memory\n");
    bzero((char *) t_tls, sizeof(*t_tls));
    t_tls->type     = htons(IP_ADDRESS_EXTENSION);
    t_tls->length   = sizeof(t_tls->settings);
    t_tls->settings = 0;

    iov[i].iov_base   = (char *) t_tls;
    iov[i].iov_len    = sizeof(t_tls->type) + sizeof(t_tls->length) + sizeof(t_tls->settings);
    len              += iov[i++].iov_len;

    SET_BITS_HIGH(t_tls->settings, (ip_ext_head->s).status, 8, 32);
    SET_BITS_MIDDLE(t_tls->settings, (ip_ext_head->s).C, 24, 1);
    SET_BITS_MIDDLE(t_tls->settings, (ip_ext_head->s).I, 23, 1);
    SET_BITS_MIDDLE(t_tls->settings, (ip_ext_head->s).L, 22, 1);
    SET_BITS_MIDDLE(t_tls->settings, (ip_ext_head->s).Q, 21, 1);
    SET_BITS_MIDDLE(t_tls->settings, (ip_ext_head->s).A, 20, 1);
    SET_BITS_MIDDLE(t_tls->settings, (ip_ext_head->s).P, 19, 1);
    SET_BITS_MIDDLE(t_tls->settings, (ip_ext_head->s).scope, 10, 3);
    SET_BITS_LOW(t_tls->settings, (ip_ext_head->s).prefix_len, 7);
    
    if((ip_ext_head->s).C) {
      iov[i].iov_base = (char *) &ip_ext_head->ip_addr;
      iov[i].iov_len  = sizeof(ip_ext_head->ip_addr);
      len            += iov[i].iov_len;
      t_tls->length  += iov[i++].iov_len;
    }

    if((ip_ext_head->s).L) {
      iov[i].iov_base = (char *) &ip_ext_head->preferred_lifetime;
      iov[i].iov_len  = sizeof(ip_ext_head->preferred_lifetime);
      len            += iov[i].iov_len;
      t_tls->length  += iov[i++].iov_len;

      iov[i].iov_base = (char *) &ip_ext_head->valid_lifetime;
      iov[i].iov_len  = sizeof(ip_ext_head->valid_lifetime);
      len            += iov[i].iov_len;
      t_tls->length  += iov[i++].iov_len;
    }

    if(ip_ext_head->dns_name[0] != '\0') {
      /* dns name is present */
      iov[i].iov_base = (char *) ip_ext_head->dns_name;
      iov[i].iov_len  = strlen(ip_ext_head->dns_name);
      len            += iov[i].iov_len;
      t_tls->length  += iov[i++].iov_len;
    }
    t_tls->length     = htons(t_tls->length);
    t_tls->settings   = htonl(t_tls->settings);
    
    *temp_tls = t_tls;
    temp_tls  = &(*temp_tls)->next;
  }

  if(writev(sockfd, &iov[0], i) != len){
    printf("Error in sending request message to client daemon. Returning...\n");
    return 0;
  }

  printf("Sending:< ");
  {
    int count;
    char *c;

    for(count = 0; count < i; count++) {
      for(c = iov[count].iov_base; iov[count].iov_len > 0; c++, iov[count].iov_len--)
	printf("%02x ", (uint8_t)*c);
      printf("\n");
    }
  }
  printf(">\n");
  /* free up the list of type length and settings we made */
  while(tls_head) {
    t_tls = tls_head;
    tls_head = tls_head->next;
    free(t_tls);
  }

#ifdef DEBUG
  printf("Waiting for reply to release msg\n");
#endif

  n = recv(sockfd, recv_msg, MAX_MSG_SIZE, 0);

  if(n < 0){
    printf("Error in recv. Returning...\n");
    return 0;
  }
  if(n == 0){
    printf("Connection closed by client daemon. Returning...\n");
    return 0;
  }

#ifdef DEBUG
  printf("Reply to release-> ");
  print_packet(recv_msg, n);
  printf("\n");

  memcpy(&header, recv_msg, sizeof(header));
  header.pid        = ntohs(header.pid);
  header.request_no = ntohl(header.request_no);

  printf("Reply from client daemon\n\n");
  printf("msg_type   = %u\n", header.msg_type);
  printf("pid        = %u\n", header.pid);
  printf("request_no = %u\n", header.request_no);
#endif

  return 1;

}
