# include <stdio.h>
# include <errno.h>
# include <signal.h>
# include <netdb.h>
# include <string.h>
# include <stdlib.h>
# include <syslog.h>

# include <sys/types.h>
# include <sys/socket.h>
# include <sys/time.h>
# include <sys/param.h>
# include <sys/file.h>
# include <sys/ioctl.h>
# include <sys/sysctl.h>
# include <sys/uio.h>

# include <netinet/in.h>

# include <arpa/inet.h>

# include <net/if.h>
# include <net/if_dl.h>
# include <net/route.h>

# include "max_dns_name.h"
# include "in6_addr_list.h"
# include "host_subnet_interface_structures.h"

# define PRINT

void insert_interface_in_list(INTERFACE *interface, INTERFACE **head) {
  interface->next = *head;
  (*head)         = interface;
}

void insert_addr_in_list(IN6_ADDR_LIST *temp_iaddr, IN6_ADDR_LIST **head) {
  temp_iaddr->next = *head;
  (*head)          = temp_iaddr;
}


INTERFACE *find_interface(uint index, INTERFACE *head){
  /*
   * Searches the link list of interface structures for
   * an entry with interface index = index. head points
   * to the head of link list which is the first entry
   * in link list.
   */
  INTERFACE *tmp = head;

  while(tmp != NULL){
    if(tmp->index == index)
      break;
    tmp = tmp->next;
  }
  return tmp;
}

void print(INTERFACE **head){
  /*
   * Prints info about those entries in link list which have a
   * link local address. Thus these are the interfaces we would
   * like to use for DHCPV6
   * This function also removes unwanted interfaces from the
   * list.
   */
  IN6_ADDR_LIST *t_addr;
  INTERFACE     *t_interface, **temp_interface = head;
  char           buf[128];
  uint8_t        i;

  while(*temp_interface){
    t_interface = *temp_interface;
    if(t_interface->link_local_addr != NULL){
      /*
       * This is the type of interface we look for (the one
       * with a link local address).
       */

#ifdef PRINT
      printf("Link local addr of [%u] name <%s> = %s\n",
	     t_interface->index,
	     t_interface->name,
	     inet_ntop(AF_INET6, t_interface->link_local_addr, buf, sizeof(buf)));
#endif
      
      t_addr = t_interface->addr_list_head;

#ifdef PRINT
      printf("\t\tIPv6 addr list ");
#endif

      while(t_addr != NULL){

#ifdef PRINT
	printf("%s ",inet_ntop(AF_INET6, & t_addr->ip_addr, buf, sizeof(buf)));
#endif

	t_addr = t_addr->next;
      }
      printf("\n");
    }
    else{
      /*
       * Delete this interface
       */
      *temp_interface = (*temp_interface)->next;
      free(t_interface);
      continue;
    }
    temp_interface = &(*temp_interface)->next;
  }
}

/*********************************************************************************/

# define ADDRESS_FAMILY AF_INET6
/*********************************************************************************/

int                mib[] = { CTL_NET, PF_ROUTE, 0, ADDRESS_FAMILY, NET_RT_IFLIST, 0 };
caddr_t            buf;
struct if_msghdr  *ifm;
struct ifa_msghdr *ifam;
struct sockaddr   *sa;
INTERFACE         *temp_interface;

/*********************************************************************************/
void discover_interfaces(INTERFACE **in_head){
  /*
   * Discovers all IPv6 interfaces and binds each
   * available address to CLIENT_PORT by opening
   * a socket per address per interface including
   * link local address.
   */
  caddr_t cp;
  int     interface_in_list;
  int     size;
  
  if (sysctl(mib, 6, NULL, &size, NULL, 0) < 0) {
    perror("sysctl0");
    exit(1);
  }

  if ((buf = malloc(size)) == NULL) {
    fprintf(stderr, "malloc failed\n");
    exit(1);
  }

  if (sysctl(mib, 6, buf, &size, NULL, 0) < 0) {
    perror("sysctl");
    exit(1);
  }

  for (cp = buf; cp < buf + size; cp += ifm->ifm_msglen) {
    ifm = (struct if_msghdr *)cp;
    ifam = (struct ifa_msghdr *)cp;
    interface_in_list = 1;
    temp_interface = find_interface(ifm->ifm_index, *in_head);

    if(temp_interface == NULL){
      temp_interface = (INTERFACE *) malloc(sizeof(INTERFACE));
      if(!temp_interface)
	errx(1, "Insufficient memory\n");
      bzero((char *)temp_interface, sizeof(*temp_interface));
      temp_interface->index = ifm->ifm_index;
      interface_in_list = 0;
    }
    
    if (ifm->ifm_type == RTM_IFINFO)
      ifinfo();
    else
      ifaddr();

    if(interface_in_list == 0){
      insert_interface_in_list(temp_interface, in_head);
      if(*in_head == NULL)
	printf("Null after inserting");
    }
  }

  free(buf);  
  print(in_head);
}
/*********************************************************************************/

#define ROUNDUP(a) \
        ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
#define ADVANCE(x, n) (x += ROUNDUP((n)->sa_len))
/*********************************************************************************/

ifinfo(){
  caddr_t cp;
  int len, i;
  
  cp = (caddr_t)(ifm + 1);
  len = ifm->ifm_msglen;
  
  for (i = 0; i < RTAX_MAX; i++)
    if (ifm->ifm_addrs & (1 << i)) {
      sa = (struct sockaddr *)cp;
      bit(i);
      sock();
      ADVANCE(cp, sa);
    }
  
  if (cp != (caddr_t)ifm + len)
    printf("delta = %d\n", cp - ((caddr_t)ifm + len));
}
/*********************************************************************************/

ifaddr(){
  caddr_t cp;
  int len, i;
  
  cp = (caddr_t)(ifam + 1);
  len = ifam->ifam_msglen;
  
  for (i = 0; i < RTAX_MAX; i++)
    if (ifam->ifam_addrs & (1 << i)) {
      sa = (struct sockaddr *)cp;
      bit(i);
      sock();
      ADVANCE(cp, sa);
    }
  
  if (cp != (caddr_t)ifam + len)
    printf("delta = %d\n", cp - ((caddr_t)ifam + len));
}
/*********************************************************************************/

bit(int i){
  switch (i) {
  case RTAX_DST:
    return;
    
  case RTAX_GATEWAY:
    return;
    
  case RTAX_NETMASK:
    return;
    
  case RTAX_GENMASK:
    return;
    
  case RTAX_IFP:
    return;
    
  case RTAX_IFA:
    return;
    
  case RTAX_AUTHOR:
    return;
    
  case RTAX_BRD:
    return;
    
  default:
    return;
  }
}
/*********************************************************************************/

sock(){
  u_char *cp;
  char buf[64];
  int i;
  IN6_ADDR_LIST *temp_iaddr;
  
  switch (sa->sa_family) {

  case AF_UNSPEC:
    break;
    
  case AF_INET:
    return;
    
  case AF_INET6:
#define SIN6(x)	((struct sockaddr_in6 *)x)
    temp_iaddr            = (IN6_ADDR_LIST *)malloc(sizeof(IN6_ADDR_LIST));
    if(!temp_iaddr)
      errx(1, "Insufficient memory. Exiting...\n");
    bzero((char *) temp_iaddr, sizeof(*temp_iaddr));

    temp_iaddr->ip_addr   = SIN6(sa)->sin6_addr;
    
    if(IN6_IS_ADDR_LOOPBACK(& temp_iaddr->ip_addr)){
      free(temp_iaddr);
    }
    else{
      if(IN6_IS_ADDR_LINKLOCAL(& temp_iaddr->ip_addr)){
	temp_interface->link_local_addr  = (struct in6_addr *)malloc(sizeof(struct in6_addr));
	if(!temp_interface->link_local_addr)
	  errx(1, "Insufficient memory. Exiting...\n");
	*temp_interface->link_local_addr = temp_iaddr->ip_addr;
	free(temp_iaddr);
      }
      else
	insert_addr_in_list(temp_iaddr, & temp_interface->addr_list_head);
    }
    return;
    
  case AF_LINK:
#define SDL(x)	((struct sockaddr_dl *)x)
   
    sprintf(temp_interface->name,"%*s",
	    SDL(sa)->sdl_nlen,
	    SDL(sa)->sdl_data);
    /*
    cp = (u_char *)(SDL(sa)->sdl_data + SDL(sa)->sdl_nlen);
    
    for (i = 0; i < SDL(sa)->sdl_alen; i++){
      temp_interface->hardware_addr.hw_addr[i] = *cp++;
      }*/
    return;
    
  default:
  }
}
